@@ -32,11 +32,11 @@ bool WeylTensor::parse(Kernel&, keyval_t& )
 	return true;
 	}
 
-void WeylTensor::validate(const Kernel& kernel, const Ex& pat) const
+void WeylTensor::validate(Kernel& kernel, std::shared_ptr<Ex> pat) const
 	{
-	if(Algorithm::number_of_indices(kernel.properties, pat.begin())!=4)
+	if(Algorithm::number_of_indices(kernel.properties, pat->begin())!=4)
 		throw ConsistencyException("WeylTensor: need exactly 4 indices.");
-	index_iterator indit=index_iterator::begin(kernel.properties, pat.begin());
+	index_iterator indit=index_iterator::begin(kernel.properties, pat->begin());
 	auto ind=kernel.properties.get<Indices>(indit, true);
 	// We cannot access the right things from parse()
 	if(ind) {
