@@ -3,6 +3,7 @@
 SelectFileDialog::SelectFileDialog(const Glib::ustring& title, Gtk::Window& parent, bool modal)
 	: Gtk::Dialog(title, parent, modal)
 	, choose("...")
+	, hbox(Gtk::Orientation::ORIENTATION_HORIZONTAL)
 	{
 	set_resizable(false);
 	entry.set_width_chars(60);
@@ -38,9 +39,9 @@ void SelectFileDialog::choose_dialog()
 	{
 	Gtk::FileChooserDialog fc("Select a file...");
 	fc.set_transient_for(*this);
-	fc.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+	fc.add_button("Cancel", Gtk::RESPONSE_CANCEL);
 	fc.add_button("Select", Gtk::RESPONSE_OK);
 
 	if (fc.run() == Gtk::RESPONSE_OK)
 		entry.set_text(fc.get_filename());
-	}
\ No newline at end of file
+	}
